
$(window).on("load", function () {
  canvas = document.getElementById('game');
  context = canvas.getContext('2d');
  canvas.width = 500;
  canvas.height = 500;
  // setTimeout(() => menu(), 250);
});

function random(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}
var canvas;
var context;
var count = 0;
var speed = 10;
var people = [];
var numberPeople = 10;
var startInfected = 1;
var sickDur = 10;
var gameOver = false;
var meth = 5;
var infectRadius = 10;
var infectRate = 10;
var stats = {
  healthy: 0,
  infected: 0,
  recovered: 0,
};
var time = 0;
var rVal = 0;
var infCount = 0;
var infFreq = 2;
var graphCount = 0;
var deathRate = 10;
var groceryStore = true;
var groceryRate = 15;
var groceryDur = 3;

function updateVars() {
  numberPeople = Number($("#numPeople").val());
  startInfected = Number($("#numInfect").val());
  sickDur = Number($("#infectDur").val());
  infectRadius = Number($("#infectRad").val());
  infectRate = Number($("#infectivity").val()) * 10;
  infFreq = Number($("#infFreq").val());
  speed = Number($("#speed").val());
  meth = Number($("#rander").val());
  deathRate = Number($("#deathrate").val());
  groceryStore = $("#groStore").is(':checked');
  groceryRate = Number($("#grorate").val());
  groceryDur = Number($("#grodur").val());
  console.log(`NEW SIMULATION VALUES: {
    number = ${numberPeople}
    infected = ${startInfected}
    duration = ${sickDur} sec
    infectRadius = ${infectRadius}px
    infectRate = ${infectRate}%
    infectFreq = every ${infFreq} frames
    speed = ${speed} tenths pixel per frame
    randomness = ${meth} 0.1% per frame
    GroceryStore = ${groceryStore}
    GroceryRate = ${groceryRate} 0.01% per frame
    GroceryDur = ${groceryDur} frames
  }`);
  stats = {
    healthy: numberPeople - startInfected,
    infected: startInfected,
    recovered: 0,
  };
  printStats();
}

class Person {
  constructor(state){
    this.pos = {
      x: random(20, 4980),
      y: random(20, 4980),
    };
    this.mov = {};
    this.mov.x = random(0, speed);
    this.mov.y = speed - this.mov.x;
    if (random(1, 4) < 3) {
      this.mov.x *= -1;
    }
    if (random(1, 4) < 3) {
      this.mov.y *= -1;
    }
    this.state = state;
    this.time = 0;
    this.spread = 0;
    this.hist = {
      x: 2500,
      y: 2500,
    };
    this.shop = 0;
  }
  get x(){
    return Math.round(this.pos.x / 10);
  }
  get y(){
    return Math.round(this.pos.y / 10);
  }
  get color(){
    switch (this.state) {
      case 0: return "#94fc03";
      case 1: return "#fc5203";
      case 2: return "#33beff";
    }
    return "#999999";
  }
  move() {
    this.pos.x = this.pos.x + this.mov.x;
    this.pos.y = this.pos.y + this.mov.y;
    if (this.pos.x < 20) {
      this.mov.x *= -1;
      this.pos.x = 19 + (20 - this.pos.x);
    } else if (this.pos.x > 4980) {
      this.mov.x *= -1;
      this.pos.x = 4981 - (this.pos.x - 4980);
    }
    if (this.pos.y < 20) {
      this.mov.y *= -1;
      this.pos.y = 19 + (20 - this.pos.y);
    } else if (this.pos.y > 4980) {
      this.mov.y *= -1;
      this.pos.y = 4981 - (this.pos.y - 4980);
    }
    this.changeCourse();
    if (groceryStore) {
      this.grocery();
    }
  }

  grocery() {
    if (this.shop > 0) {
      this.shop++;
      if (this.shop > groceryDur) {
        this.shop = 0;
        this.pos.x = this.hist.x;
        this.pos.y = this.hist.y;
      }
    } else {
      if (random(0, 9999) < groceryRate) {
        this.shop = 1;
        this.hist.x = this.pos.x;
        this.hist.y = this.pos.y;
        this.pos.x = 2500;
        this.pos.y = 2500;
      }
    }
  }
  changeCourse() {
    if (random(1,1000) > (1000 - meth)) {
      this.mov.x = random(0, speed);
      this.mov.y = speed - this.mov.x;
      if (random(1, 4) < 3) {
        this.mov.x *= -1;
      }
      if (random(1, 4) < 3) {
        this.mov.y *= -1;
      }
    }
  }
  infect() {
    if (this.state != 1) {
      return;
    }
    this.time++;
    this.spread += infectionCheck(this.pos.x, this.pos.y);
    if (this.time > sickDur * 30) {
      this.state = 2;
    }
  }
}

function infectionCheck(x, y) {
  let count = 0;
  for (const person of people) {
    if (person.state === 0) {
      const dx = Math.abs(person.pos.x - x);
      const dy = Math.abs(person.pos.y - y);
      const dist = Math.hypot(dx, dy);
      if (dist <= infectRadius * 10) {
        // console.log("within radius!");
        if (random(0, 999) < infectRate) {
          person.state = 1;
          count++;
        }
      }
    }
  }
  return count;
}

function start() {
  replace("#startbox", '<button onclick="pause()">UN/PAUSE</button> <button onclick="over()">STOP</button>');
  updateVars();
  $("#grapha").empty();
  $("#graphb").empty();
  $("#graphc").empty();
  for (let i = 0; i < numberPeople; i++) {
    const state = (i < startInfected) ? 1 : 0;
    people.push(new Person(state));
  }
  gameOver = false;
  loop();
}

function pause() {
  if (gameOver) {
    gameOver = false;
    loop();
  } else {
    gameOver = true;
  }
}

function over() {
  gameOver = true;
  people = [];
  context.clearRect(0, 0, canvas.width, canvas.height);
  replace("#startbox", `<button onclick="start()">START</button>`);
  replace("#stats", "Stats will appear when running a simulation");
  stats = {
    healthy: 0,
    infected: 0,
    recovered: 0,
  };
  time = 0;
  rVal = 0;
  infCount = 0;
  graphCount = 0;
}

function draw() {
  for (const person of people) {
    context.beginPath();
    context.arc(person.x, person.y, 2,0,2*Math.PI);
    context.lineWidth = 5;
    context.strokeStyle = person.color;
    context.stroke();
    if (person.state === 1) {
      context.beginPath();
      context.arc(person.x, person.y, infectRadius,0,2*Math.PI);
      context.lineWidth = 1;
      context.strokeStyle = "#ffed4f";
      context.stroke();
    }
  }
  if (groceryStore) {
    context.beginPath();
    context.rect(243, 243, 14, 14);
    context.lineWidth = 1;
    context.strokeStyle = "#eee";
    context.stroke();
  }
}

function moveThem() {
  for (const person of people) {
    person.move();
  }
}

function infection() {
  for (const person of people) {
    person.infect();
  }
}

function calcStats() {
  stats = {
    healthy: 0,
    infected: 0,
    recovered: 0,
  };
  let spread = 0;
  for (const person of people) {
    switch (person.state) {
      case 0:
        stats.healthy += 1;
        break;
      case 1:
        stats.infected += 1;
        spread += ((sickDur * 30) / person.time) * person.spread;
        break;
      case 2:
        stats.recovered += 1;
        break;
    }
  }
  if (!isNaN(spread / stats.infected)){
    rVal = spread / stats.infected;
  }
}


function printStats() {
  const hp = Math.round(stats.healthy / numberPeople * 100);
  const ip = Math.round(stats.infected / numberPeople * 100);
  const rp = Math.round(stats.recovered / numberPeople * 100);
  const r1 = Math.floor(rVal);
  const r2 = Math.round((rVal - r1)*100);
  const dead = Math.round(stats.recovered * (deathRate / 1000));
  const out = `Healthy: ${stats.healthy} (${hp}%) | Infected: ${stats.infected} (${ip}%) | Recovered: ${stats.recovered} (${rp}%)<br>Time: ${Math.floor(time / 30)} seconds | R Value: ${r1}.${r2} | Dead: ${dead}`;
  replace("#stats", out);
}
function printGraph() {
  if (stats.infected > 0) {
    const hp = Math.round((stats.healthy / numberPeople * 100));
    let graph = `<div class="graphElementH" style="height: ${hp}%;"></div>`;
    append("#grapha", graph);
    const ip = Math.round((stats.infected / numberPeople * 100)) ;
    graph = `<div class="graphElement" style="height: ${ip}%;"></div>`;
    append("#graphb", graph);
    const rp = Math.round((stats.recovered / numberPeople * 100));
    graph = `<div class="graphElementR" style="height: ${rp}%;"></div>`;
    append("#graphc", graph);
  }
}

function loop() {
  if (!gameOver) {
    requestAnimationFrame(loop);
  }
  // slow game loop to 30 fps instead of 60 (60/2 = 30)
  if (++count < 2) {
    return;
  }
  count = 0;
  // actually do stuffs
  moveThem();
  infCount++;
  if (infCount >= infFreq) {
    infection();
    calcStats();
    printStats();
    infCount = 0;
  }
  graphCount++;
  if (graphCount >= 3) {
    printGraph();
    graphCount = 0;
  }
  time++;
  // clear the canvas
  context.clearRect(0, 0, canvas.width, canvas.height);
  // draw the dots
  draw();
}

// appends supplied twee to target element
function append(target, content){
  content = content.replace(/\n/g, " ");
  if ("string" !== typeof target) {
    console.log(`Invalid target type sent to aw.append! (${typeof target})`);
    return;
  }
  const x = target.slice(0, 1);
  if (x !== "#" && x !== ".") {
    console.log(`Invalid target string sent to aw.append! (${target})`);
  }
  $(target).append(content);
}

// empties target element and appends twee
function replace(target, content) {
  content = content.replace(/\n/g, " ");
  if ("string" !== typeof target) {
    console.log(`Invalid target type sent to aw.replace! (${typeof target})`);
    return;
  }
  const x = target.slice(0, 1);
  if (x !== "#" && x !== ".") {
    console.log(`Invalid target string sent to aw.replace! (${target})`);
  }
  $(target).empty();
  $(target).append(content);
}

